open! Core
open! Bonsai_web

(** This is a notifications component that provides pretty and user-friendly(ish)
    notifications, with automatic fade-out & dismissal after a time span. *)

type t

val create
  :  ?dismiss_notifications_after:Time_ns.Span.t Value.t (** default = 15s *)
  -> ?dismiss_errors_automatically:bool Value.t (** default = false *)
  -> Source_code_position.t
  -> t Computation.t

(** [add_error] will create a notification with [text] in primary focus and [error] pretty
    printed in small font (if provided). *)
val add_error : ?error:Error.t -> t -> text:string -> unit Ui_effect.t

(** [add_success] creates a notification with only [text] in primary focus. *)
val add_success : t -> text:string -> unit Ui_effect.t

module Notification_style : sig
  module type S = sig
    (** These CSS classes are applied to the notifications that are generated by
        [to_vdom]. To customise the color of these notifications, set the
        [background-color] attribute to your desired color. *)

    val error : string
    val success : string
  end

  type t = (module S)
end

(** [to_vdom] constructs the vdom necessary to display these notifications. The styling
    required to display these notifications is handled for you via ppx_css. *)
val to_vdom
  :  ?notification_style:Notification_style.t
  -> ?notification_extra_attr:Vdom.Attr.t
  -> t
  -> Vdom.Node.t
